//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x000000,//背景色
  },
}
//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Title: "assets/title-scene.png",
  Game: "assets/game-scene.png"
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  
  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new TitleScene());
  }
}

//タイトルシーン
class TitleScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵(取りあえず)
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Title].texture;
    this.addChild(img)

    const style = new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 40,
      fill: 0xffffff,
      fontWeight: 'bold',
      stroke: 0xFF0000,//縁取りの色
      strokeThickness: 4,//縁取りの太さ
    });
    const p = new PIXI.Text("おばけ", style);
    p.anchor.set(0.5, 0.5);
    p.position.set(Config.Screen.Width * 0.5, Config.Screen.Height * 0.15);
    this.addChild(p);

    this.isFading = true; 

    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 

      new DQ3Out(this, 0x000000, 8, 2, () => {
        core.replaceScene(new MainScene());
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}


//メインシーン
class MainScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Game].texture;
    this.addChild(img)

    this.isFading = true; 

    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 

      new DQ3Out(this, 0x000000, 8, 2, () => {
        core.replaceScene(new TitleScene());
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}

//ドラクエ３風
class DQ3Out extends Graphics {
  constructor(container, color, size, frequency, callback){
    super();
    container.addChild(this);
    this.size = size;
    this.color = color;
    this.frequency = frequency;
    this.radius = Config.Screen.Width / size | 0;//
    this.center_x = Config.Screen.Width * 0.5;
    this.center_y = Config.Screen.Height * 0.5;
    this.rad = 0;
    this.count = 0;
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }

    //テクスチャー生成用
    this.renderTexture1 = new PIXI.RenderTexture.create({width: Config.Screen.Width * 0.5, height: Config.Screen.Height * 0.5});
    this.renderTexture2 = new PIXI.RenderTexture.create({width: Config.Screen.Width * 0.5, height: Config.Screen.Height * 0.5});
    this.renderFlip = true;

    //コピー用のスプライト
    this.pics = [];
    for(let i = 0; i < 3; i++){
      const p = new PIXI.Sprite();
      p.rotation = Math.PI * 0.5 * (i+1);//90度ずつ回転
      p.position.set(Config.Screen.Width * 0.5, Config.Screen.Height * 0.5);
      this.addChild(p);
      this.pics[i] = p;
    }

    //状況保存用の配列
    this.dots = [];
    for(let i = 0; i < this.radius; i++){
      this.dots[i] = [];
      for(let j = 0; j < this.radius; j++){
        this.dots[i][j] = false;
      }
    }
  } 
  update(delta){
    super.update(delta);
    if(this.count++ < this.frequency) return;
    this.count = 0;
    this.rad += 0.1;
    if(this.rad > Math.PI * 0.5){
      this.onFinished();
      this.destroy();
      return;
    }
    //ドットを描く
    for(let x = 0; x < this.radius; x++){
      const xx = Math.cos(this.rad) * x | 0;
      const yy = Math.sin(this.rad) * x | 0;
      if(!this.dots[yy][xx]){//無いところに
        this.rectFill(xx * this.size + this.center_x, yy * this.size + this.center_y, this.size, this.size, this.color);
        this.dots[yy][xx] = true;
        //隙間があったら埋める処理
        const dots = this.dots[yy];
        for(let i = xx; i < this.radius; i++){
          if(!dots[i]){
            this.rectFill(i * this.size + this.center_x, yy * this.size + this.center_y, this.size, this.size, this.color);
            dots[i] = true;
          }
        }
      }
    }
    //2つのテクスチャーを交互に使う
    this.x = -Config.Screen.Width * 0.5;
    this.y = -Config.Screen.Height * 0.5;
    if(this.renderFlip = !this.renderFlip){
      core.app.renderer.render(this, this.renderTexture1);
      for(let i = 0; i < 3; i++){
        this.pics[i].texture = this.renderTexture1;
      }
    }else{
      core.app.renderer.render(this, this.renderTexture2);
      for(let i = 0; i < 3; i++){
        this.pics[i].texture = this.renderTexture2;
      }
    }
    this.x = 0;
    this.y = 0;
  }
}

class FadeIn extends Graphics {
  constructor(container, color, frameLength, callback){
    super();
    container.addChild(this);
    this.rectFill(0, 0, Config.Screen.Width, Config.Screen.Height, color);
    this.step = 1 / frameLength;
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  update(delta){
    super.update(delta);
    this.alpha -= this.step;
    if(this.alpha <= 0){
      this.onFinished();
      this.destroy();
      return;
    }   
  }
}
